/******************************************************************************\
*                                                                              *
* 20/4/2009                                                                    *
* Tweener - Source File                                                        *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Animation/Tweener.h>
#include <Animation/Tweenable.h>
#include <QTimerEvent>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

Tweener::Tweener():
	m_pNext(NULL),
	m_timerID(-1),
	m_interval(1000.0f/25.0f),
	m_nmbTweenable(0)
{

}

Tweener::~Tweener()
{

}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//ADD TWEENABLE
void Tweener::AddTweenable(Tweenable *tweenable)
{
	//connect slots
	connect(this,SIGNAL(UpdateTweenable(float)),tweenable,SLOT(Tween(float)));
	connect(this,SIGNAL(KillTweenable()),tweenable,SLOT(Destroy()));
	
	//inc
	++m_nmbTweenable;
}

//START TWEEN
void Tweener::StartTween()
{
	//reset
	m_elapsed = 0;
		
	//if, clear old running timer
	if(m_timerID != -1)
	{
		killTimer(m_timerID);
	}
	
	//start new timer
	m_timerID = startTimer(m_interval);

	OnStart();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

//TIMER EVENT
void Tweener::timerEvent( QTimerEvent * event)
{
	//update
	m_elapsed += m_interval;
	
	if(m_elapsed <= m_duration)
	{
		if(!m_singleShot)
		{
			emit UpdateTweenable(m_elapsed/m_duration);
		}
	}
	else
	{
		emit UpdateTweenable(1.0f);
		emit KillTweenable();

		//reset
		killTimer(m_timerID);
		m_timerID = -1;
		m_nmbTweenable = 0;

		OnEnd();

		//check if pending
		if(m_pNext)
		{
			m_pNext->StartTween();
		}
	}
	
	
	//accept
	event->accept();
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/