/******************************************************************************\
*                                                                              *
* 4/4/2009                                                                     *
* Graph - Source File                                                          *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Graph/Graph.h>
#include <Graph/DataNode.h>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

Graph::Graph(const vector<DataNode> &nodes, const vector<std::vector<bool> > &relations) :
	m_nodes(nodes),
	m_relations(relations)
{
	
}

Graph::~Graph()
{

}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

DataNode & Graph::GetRootNode()
{
	return m_nodes[0];
}

//Return all outgoing nodes
void Graph::GetNodesOut(const DataNode& node, vector<DataNode*> &outNodes ) 
{
	for(int i=0; i<m_relations.size(); i++) 
	{
		if(m_relations[node.GetId()][i])
		{
			outNodes.push_back(&m_nodes[i]);
		}
	}
}

//Return all incoming nodes
void Graph::GetNodesIn(const DataNode& node, vector<DataNode*> &inNodes) 
{
	for(int i=0; i<m_relations.size(); i++) 
	{
		if(m_relations[i][node.GetId()])
		{
			inNodes.push_back(&m_nodes[i]);
		}
	}
}

int Graph::GetLinkDirection(DataNode &first, DataNode &second)
{
	// 0 = incoming, 1 = outgoing, 2=bidirectional
	if(m_relations[second.GetId()][first.GetId()] && m_relations[first.GetId()][second.GetId()])
	{
		return 2;
	}
	else if(m_relations[first.GetId()][second.GetId()])
	{
		return 0;
	}
	else
	{
		return 1;
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/