#ifndef NODESORT_H
#define NODESORT_H

/******************************************************************************\
*                                                                              *
* 27/5/2009                                                                    *
* NodeSort - Header File                                                       *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/
/******************************************************************************\
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX DESCRIPTION XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
\******************************************************************************/

//REQUIRED HEADERS 

#include <Definitions.h>
#include <Graph/DataNode.h>
#include <Math/Vector2.h>
#include <TreeNodeRep.h>

//DECLARATIONS


//PUBLIC DEFINITIONS TYPEDEFS
//Sort data nodes
bool SortDataNodeNameAsc(DataNode *first, DataNode *second)
{
	std::string strFirst = first->GetName().toLower().toStdString();
	std::string strSecond = second->GetName().toLower().toStdString();
	if(strFirst < strSecond)
	{
		return true;
	}
	else 
	{
		return false;
	}
}

bool SortDataNodeNameDesc(DataNode *first, DataNode *second)
{
	return !SortDataNodeNameAsc(first, second);
}

bool SortDataLinkInAsc(DataNode *first, DataNode *second)
{
	int inFirst, outFirst, biFirst;
	int inSecond, outSecond, biSecond;
	Tree::s_Tree->GetRelations(*first, inFirst, outFirst, biFirst);
	Tree::s_Tree->GetRelations(*second, inSecond, outSecond, biSecond);
	if(inFirst < inSecond)
	{
		return false;
	}
	if(inFirst > inSecond)
	{
		return true;
	}
	else
	{
		return SortDataNodeNameAsc(first, second);
	}
}

bool SortDataLinkInDesc(DataNode *first, DataNode *second)
{
	int inFirst, outFirst, biFirst;
	int inSecond, outSecond, biSecond;
	Tree::s_Tree->GetRelations(*first, inFirst, outFirst, biFirst);
	Tree::s_Tree->GetRelations(*second, inSecond, outSecond, biSecond);
	if(inFirst > inSecond)
	{
		return false;
	}
	if(inFirst < inSecond)
	{
		return true;
	}
	else
	{
		return SortDataNodeNameAsc(first, second);
	}
}

bool SortDataLinkOutAsc(DataNode *first, DataNode *second)
{
	int inFirst, outFirst, biFirst;
	int inSecond, outSecond, biSecond;
	Tree::s_Tree->GetRelations(*first, inFirst, outFirst, biFirst);
	Tree::s_Tree->GetRelations(*second, inSecond, outSecond, biSecond);
	if(outFirst < outSecond)
	{
		return false;
	}
	if(outFirst > outSecond)
	{
		return true;
	}
	else
	{
		return SortDataNodeNameAsc(first, second);
	}
}

bool SortDataLinkOutDesc(DataNode *first, DataNode *second)
{
	int inFirst, outFirst, biFirst;
	int inSecond, outSecond, biSecond;
	Tree::s_Tree->GetRelations(*first, inFirst, outFirst, biFirst);
	Tree::s_Tree->GetRelations(*second, inSecond, outSecond, biSecond);
	if(outFirst > outSecond)
	{
		return false;
	}
	if(outFirst < outSecond)
	{
		return true;
	}
	else
	{
		return SortDataNodeNameAsc(first, second);
	}
}

bool SortDataLinkBiAsc(DataNode *first, DataNode *second)
{
	int inFirst, outFirst, biFirst;
	int inSecond, outSecond, biSecond;
	Tree::s_Tree->GetRelations(*first, inFirst, outFirst, biFirst);
	Tree::s_Tree->GetRelations(*second, inSecond, outSecond, biSecond);
	if(biFirst < biSecond)
	{
		return false;
	}
	if(biFirst > biSecond)
	{
		return true;
	}
	else
	{
		return SortDataNodeNameAsc(first, second);
	}
}

bool SortDataLinkBiDesc(DataNode *first, DataNode *second)
{
	int inFirst, outFirst, biFirst;
	int inSecond, outSecond, biSecond;
	Tree::s_Tree->GetRelations(*first, inFirst, outFirst, biFirst);
	Tree::s_Tree->GetRelations(*second, inSecond, outSecond, biSecond);
	if(biFirst > biSecond)
	{
		return false;
	}
	if(biFirst < biSecond)
	{
		return true;
	}
	else
	{
		return SortDataNodeNameAsc(first, second);
	}
}

//Tree Node sorting
bool SortNodeNameAsc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataNodeNameAsc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortNodeNameDesc(TreeNode *first, TreeNode *second)
{
	return !SortNodeNameAsc(first, second);
}

bool SortLinkInAsc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataLinkInAsc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortLinkInDesc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataLinkInDesc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortLinkOutAsc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataLinkOutAsc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortLinkOutDesc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataLinkOutDesc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortLinkBiAsc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataLinkBiAsc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortLinkBiDesc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		return SortDataLinkBiDesc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
	}
}

bool SortLinkAsc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		int firstLinkDir = firstImpl->GetTree().GetLinkDirection(firstImpl);
		int secondLinkDir = secondImpl->GetTree().GetLinkDirection(secondImpl);
		if(firstLinkDir < secondLinkDir)
		{
			return false;
		}
		else if(firstLinkDir > secondLinkDir)
		{
			return true;
		}
		else 
		{
			return SortDataLinkBiAsc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
		}
	}
}

bool SortLinkDesc(TreeNode *first, TreeNode *second)
{
	if(first->GetType() == TYPE_NODE_MOVEDTO)
	{
		return false;
	}
	else if(second->GetType() == TYPE_NODE_MOVEDTO)
	{
		return true;
	}
	else 
	{
		TreeNodeImpl *firstImpl = static_cast<TreeNodeImpl *>(first);
		TreeNodeImpl *secondImpl = static_cast<TreeNodeImpl *>(second);
		int firstLinkDir = firstImpl->GetTree().GetLinkDirection(firstImpl);
		int secondLinkDir = secondImpl->GetTree().GetLinkDirection(secondImpl);
		if(firstLinkDir > secondLinkDir)
		{
			return false;
		}
		else if(firstLinkDir < secondLinkDir)
		{
			return true;
		}
		else 
		{
			return SortDataLinkBiAsc(&firstImpl->GetDataNode(), &secondImpl->GetDataNode());
		}
	}
}


/******************************************************************************\
* CLASS                                                                        *
\******************************************************************************/


#endif //NODESORT_H