#ifndef TREE_H
#define TREE_H

/******************************************************************************\
*                                                                              *
* 15/4/2009                                                                    *
* Tree - Header File                                                           *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/
/******************************************************************************\
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX DESCRIPTION XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
\******************************************************************************/

//REQUIRED HEADERS 

#include <Definitions.h>
#include <Graph/TreeNodeCatalog.h>
#include <Graph/TreeNodeImpl.h>
#include <Math/Vector2.h>

#include <list>
#include <map>

//DECLARATIONS

class Graph;
class DataNode;
class TreePlus;
class Tweener;

//PUBLIC DEFINITIONS TYPEDEFS

//

/******************************************************************************\
* CLASS                                                                        *
\******************************************************************************/
class Tree 
{
	public:

		//CLASS STATIC DEFINITIONS
		static Tree *s_Tree;
		

		//ENUM //VAR //METHODS 

		//METHODS

		Tree(Graph &graph, TreePlus &app);
		~Tree();

		//MEMBER

		void ExpandNode(TreeNodeImpl *node, bool isResetAnim=true);
		void ExploreNode(TreeNodeImpl *node);
		TreeNode * GetRoot() {return m_pRoot;}
		bool IsLeafNode(TreeNode *node);
		bool IsHasLeftSibling(TreeNode *node);
		bool IsHasRightSibling(TreeNode *node);
		TreeNode * GetParent(TreeNode *node);
		TreeNode * GetLeftSibling(TreeNode *node);
		TreeNode * GetRightSibling(TreeNode *node);
		TreeNode * GetFirstChild(TreeNode *node);
		TreeNode * GetLastChild(TreeNode *node);
		TreeNode * GetLeftNeighbor(TreeNode *node);
		TreeNode * GetLeftmost(TreeNode *node, int depth);
		void GetAllNodesForLevel(TreeNode *node, int level, std::list<TreeNode *>& nodes);
		int GetMaxDepth() {return m_maxDepth;}
		int GetLevel(TreeNode *node) {return m_level[node];}
		void GetChildren(TreeNode *node, std::list<TreeNode *>& children);
		void SetPosition(TreeNode *node, float x, float y);
		void SetRootNode(TreeNode *root);
		TreeNodeCatalog & GetTreeNodeCatalog() {return m_nodeCatalog;}
		void SetLayout();
		void RemoveChildren(TreeNode *node);
		void RemoveNode(TreeNode *node);
		void StartAnimation(TreeNode *startNode);
		TreeNodeImpl * GetActualNode() {return m_actualNode;}
		void UpdateLayout();
		void Sort(TreeNode *node);
		void SortChildren(std::list<TreeNode *>& children);
		void SortDataNode(std::vector<DataNode *>& nodes);
		void SetSortOrder(int sortBy, int orderBy);
		void SetAnimationDuration(int ms);
		void RedrawNodes();
		void GetRelations(DataNode& node, int &inNodesNum, int &outNodesNum, int &biNodesNum);
		void SetTooltip(DataNode& dataNode, NodeRep& node);
		int GetLinkDirection(TreeNodeImpl *node);
		void SetAnimateView(bool tf){m_bAnimateCamera = tf;}
	protected:

		//CLASS STATIC DEFINITIONS

		//ENUM //VAR //METHODS

		//METHODS

		//MEMBER

	private:

		//METHODS
		void AddChild(TreeNode *node, DataNode& childData);
		bool IsChildParentOfNode(TreeNode *node, TreeNode *child);
		void SetLevel(TreeNode *node, int level);
		void SetMovedTo(TreeNodeImpl *oldParent, TreeNodeImpl *node);
		void RemoveFromMovedToNode(TreeNode *node);
		void RemoveMovedTo(TreeNodeImpl *node);
						
		//MEMBER

		TreePlus &m_app;
		Graph &m_graph;
		TreeNode *m_pRoot;
		TreeNodeImpl *m_actualNode;

		TreeNodeCatalog m_nodeCatalog;
		std::map<TreeNode *, int> m_level;
		std::map<TreeNodeImpl *, std::list<TreeNodeMovedTo *>> m_movedToParents;
		int m_maxDepth;

		Tweener *m_fixedTweener;
		Tweener *m_moveTweener;
		Tweener *m_newTweener;
		Tweener *m_finishTweener;
		Tweener *m_rootTweenerStep1;
		Tweener *m_rootTweenerStep2;
		Tweener *m_rootTweenerStep3;
		Tweener *m_cameraTweener;
		int m_animationDuration;

		Vector2i m_boundTL;
		Vector2i m_boundBR;

		int m_orderBy;
		int m_sortBy;

		bool m_bAnimateCamera;
}; 

#endif //TREE_H