/******************************************************************************\
*                                                                              *
* 1/4/2009                                                                     *
* TreeNode - Source File                                                       *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Graph/TreeNode.h>
#include <Graph/Tree.h>
#include <Graph/TreeLink.h>
#include <TreeLinkRep.h>

#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

TreeNode::TreeNode(const DataNode &data, Tree &tree, TreeNode *parent) : 
	m_DataNode(data),
	m_tree(tree),
	m_parentNode(parent),
	m_linkToParent(NULL)
{
	m_pNodeRep = new TreeNodeRep();
	m_pNodeRep->SetTreeNode(this);
	m_pNodeRep->SetTitle(m_DataNode.GetName());

	if(parent != NULL) 
	{
		m_linkToParent = new TreeLink(this, parent);
	}
}

TreeNode::~TreeNode()
{
	if(!m_pNodeRep->scene())
	{
		delete m_pNodeRep;
	}
	if(m_linkToParent != NULL)
	{
		delete m_linkToParent;
	}
}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//ON NODE DOUBLE CLICK
void TreeNode::OnNodeDoubleClick()
{
	m_tree.ExpandNode(this);
	m_tree.SetLayout();
	m_tree.StartAnimation(this);
	m_pNodeRep->SetState(TreeNodeRep::eEXPLORE);
	m_pNodeRep->update();
}

//ON NODE CLICK
void TreeNode::OnNodeClick(TreeNodeRep *previous)
{
	if(previous)
	{
		if(previous->GetTreeNode() == this)
		{
			return;
		}
		else
		{
			previous->SetState(TreeNodeRep::eNORMAL);
		}
	}
	m_tree.ExploreNode(this);
	m_pNodeRep->SetState(TreeNodeRep::eEXPLORE);
	m_pNodeRep->update();
}

//ON NODE SHAPE CHANGE
void TreeNode::OnNodeShapeChange()
{

}

//ON NODE RIGHT CLICK
void TreeNode::OnNodeRightClick()
{

}

void TreeNode::AddChild(TreeNode *node)
{
	m_children.push_back(node);
}

void TreeNode::RemoveChild(TreeNode *node)
{
	m_children.remove(node);
}

void TreeNode::SetParentNode(TreeNode *parent)
{
	if(m_parentNode)
	{
		m_parentNode->RemoveChild(this);
	}
	m_parentNode = parent;
	if(m_parentNode != NULL)
	{
		m_linkToParent->SetTreeNodeTo(parent);
	}
	else 
	{
		delete m_linkToParent;
		m_linkToParent = NULL;
	}
}

void TreeNode::GetChildren(list<TreeNode *>& children)
{
	for(list<TreeNode *>::iterator it = m_children.begin(); it != m_children.end(); it++)
	{
		children.push_back(*it);
	}
}

void TreeNode::SetPosition( float x, float y)
{
	m_pNodeRep->setPos(x,y);

	if(m_linkToParent)
	{
		m_linkToParent->GetTreeLinkRep().SetPosition();
	}

	//not sure ??
	for(list<TreeNode *>::iterator it = m_children.begin(); it != m_children.end(); it++)
	{
		(*it)->GetLinkToParent()->GetTreeLinkRep().SetPosition();
	}
}

Vector2f TreeNode::GetPosition()
{
	return Vector2f(m_pNodeRep->pos().x(),m_pNodeRep->pos().y());
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/