/******************************************************************************\
*                                                                              *
* 1/4/2009                                                                     *
* TreeNode - Source File                                                       *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Graph/TreeNodeImpl.h>
#include <Graph/TreeNodeMovedTo.h>
#include <Graph/Tree.h>
#include <Graph/TreeLink.h>
#include <TreeLinkRep.h>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

TreeNodeImpl::TreeNodeImpl(DataNode &data, Tree &tree, TreeNode *parent) : 
	m_DataNode(data),
	m_tree(tree),
	m_parentNode(parent),
	m_linkToParent(NULL)
{
	m_pNodeRep = new TreeNodeRep();
	m_pNodeRep->SetTreeNode(this);
	m_pNodeRep->SetTitle(m_DataNode.GetName());
	m_tree.SetTooltip(m_DataNode, *m_pNodeRep);

	if(parent != NULL) 
	{
		m_linkToParent = new TreeLink(this, parent);
	}
}

TreeNodeImpl::~TreeNodeImpl()
{
	if(m_pNodeRep != NULL)
	{
		delete m_pNodeRep;
	}
	if(m_linkToParent != NULL)
	{
		delete m_linkToParent;
	}
}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//ON NODE DOUBLE CLICK
void TreeNodeImpl::OnNodeDoubleClick()
{
	m_tree.ExpandNode(this);
	m_tree.SetLayout();
	m_tree.StartAnimation(this);
	m_pNodeRep->SetState(TreeNodeRep::eEXPLORE);
	m_pNodeRep->update();
}

//ON NODE CLICK
void TreeNodeImpl::OnNodeClick(NodeRep *prev)
{
	TreeNodeRep *previous = static_cast<TreeNodeRep *>(prev);
	if(previous)
	{
		previous->SetState(TreeNodeRep::eNORMAL);
	}
	m_tree.ExploreNode(this);
	m_pNodeRep->SetState(TreeNodeRep::eEXPLORE);
	m_pNodeRep->update();
}

//ON NODE SHAPE CHANGE
void TreeNodeImpl::OnNodeShapeChange()
{

}

//ON NODE RIGHT CLICK
void TreeNodeImpl::OnNodeRightClick()
{

}

//ON NODE HOVER ENTER
void TreeNodeImpl::OnNodeHoverEnter()
{

}

//ON NODE HOVER LEAVE
void TreeNodeImpl::OnNodeHoverLeave()
{

}

void TreeNodeImpl::AddChild(TreeNode *node)
{
	m_children.push_back(node);
}

void TreeNodeImpl::RemoveChild(TreeNode *node)
{
	m_children.remove(node);
}

void TreeNodeImpl::SetParentNode(TreeNode *parent)
{
	if(m_parentNode)
	{
		m_parentNode->RemoveChild(this);
	}
	m_parentNode = parent;
	if(m_parentNode != NULL)
	{
		m_linkToParent->SetTreeNodeTo(parent);
	}
	else 
	{
		delete m_linkToParent;
		m_linkToParent = NULL;
	}
}

void TreeNodeImpl::GetChildren(list<TreeNode *>& children)
{
	for(list<TreeNode *>::iterator it = m_children.begin(); it != m_children.end(); it++)
	{
		children.push_back(*it);
	}
}

void TreeNodeImpl::SetPosition( float x, float y)
{
	m_pNodeRep->setPos(x,y);

	if(m_linkToParent)
	{
		m_linkToParent->GetTreeLinkRep().SetPosition();
	}

	//not sure ??
	for(list<TreeNode *>::iterator it = m_children.begin(); it != m_children.end(); it++)
	{
		(*it)->GetLinkToParent()->GetTreeLinkRep().SetPosition();
	}
}

Vector2f TreeNodeImpl::GetPosition()
{
	return Vector2f(m_pNodeRep->pos().x(),m_pNodeRep->pos().y());
}

TreeNodeMovedTo * TreeNodeImpl::GetNodeMovedTo()
{
	//not sure ??
	for(list<TreeNode *>::iterator it = m_children.begin(); it != m_children.end(); it++)
	{
		if((*it)->GetType() == TYPE_NODE_MOVEDTO)
		{
			return static_cast<TreeNodeMovedTo *>(*it);
		}
	}
	return NULL;
}

void TreeNodeImpl::SortChildren()
{
	m_tree.SortChildren(m_children);
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/