/******************************************************************************\
*                                                                              *
* 26/5/2009                                                                    *
* TreeNodeMovedTo - Source File                                                *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Graph/TreeNodeMovedTo.h>
#include <Graph/Tree.h>
#include <Graph/TreeLink.h>
#include <TreeLinkRep.h>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

TreeNodeMovedTo::TreeNodeMovedTo(Tree &tree, TreeNode *parent, int id) : 
	m_tree(tree),
	m_id(id),
	m_parentNode(parent),
	m_linkToParent(NULL)
{
	m_pNodeRep = new TreeNodeRep();
	m_pNodeRep->SetTreeNode(this);
	m_pNodeRep->SetTitle(QString("moved to"));

	if(parent != NULL) 
	{
		m_linkToParent = new TreeLink(this, parent);
	}
}

TreeNodeMovedTo::~TreeNodeMovedTo()
{
	if(m_pNodeRep != NULL)
	{
		delete m_pNodeRep;
	}
	if(m_linkToParent != NULL)
	{
		delete m_linkToParent;
	}
}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

void TreeNodeMovedTo::SetPosition( float x, float y)
{
	m_pNodeRep->setPos(x,y);

	if(m_linkToParent)
	{
		m_linkToParent->GetTreeLinkRep().SetPosition();
	}
}

Vector2f TreeNodeMovedTo::GetPosition()
{
	return Vector2f(m_pNodeRep->pos().x(),m_pNodeRep->pos().y());
}

void TreeNodeMovedTo::AddMovedToNodes(TreeNodeImpl *node)
{
	m_movedTo.push_back(node);
}

void TreeNodeMovedTo::RemoveFromMovedToNodes(TreeNodeImpl *node)
{
	m_movedTo.remove(node);
}

//ON NODE HOVER ENTER
void TreeNodeMovedTo::OnNodeHoverEnter()
{
	//cout << "hover enter: " << this->m_movedTo.size() << endl;
	map<int, TreeNode*> catalog = m_tree.GetTreeNodeCatalog().GetAll();
	for(map<int, TreeNode*>::iterator it = catalog.begin(); it != catalog.end(); it++)
	{
		it->second->GetNodeRep().SetState(NodeRep::eNORMAL);
	}
	for(list<TreeNodeImpl *>::iterator it = m_movedTo.begin(); it != m_movedTo.end(); it++) 
	{
		(*it)->GetNodeRep().SetState(NodeRep::eMOVEDTO);
	}
}

//ON NODE HOVER LEAVE
void TreeNodeMovedTo::OnNodeHoverLeave()
{
	//cout << "hover leave" << endl;
	m_tree.ExploreNode(m_tree.GetActualNode());
	m_tree.GetActualNode()->GetNodeRep().SetState(NodeRep::eEXPLORE);
}