/******************************************************************************\
*                                                                              *
* 15/4/2009                                                                    *
* TreeLayout - Source File                                                     *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Layout/TreeLayout.h>
#include <TreeNodeRep.h>
#include <Graph/Tree.h>
#include <list>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

float TreeLayout::m_siblingSeparation = 10.0f;
float TreeLayout::m_levelSeparation = 400.0f;
float TreeLayout::m_subtreeSeparation = 40.0f;

//METHODS

void TreeLayout::SetSiblingSeparation(float value)
{
	m_siblingSeparation = value;
}

void TreeLayout::SetLevelSeparation(float value)
{
	m_levelSeparation = value;
}

void TreeLayout::SetSubtreeSeparation(float value)
{
	m_subtreeSeparation = value;
}

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

TreeLayout::TreeLayout() :
	m_tree(NULL),
	m_xTopAdjustment(-200.0f),
	m_yTopAdjustment(0.0f)
{
	
}

TreeLayout::~TreeLayout()
{
	
}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

bool TreeLayout::Position(Tree &tree)
{
	m_tree = &tree;
	m_modifier.clear();
	m_prelim.clear();

	if(m_tree->GetRoot() == NULL) 
	{
		return true;
	}

	Vector2f oldRootPos(tree.GetRoot()->GetPosition());
	Firstwalk(tree.GetRoot(), 0);
	float offset =  oldRootPos[1] - m_prelim[tree.GetRoot()];
	return Secondwalk(tree.GetRoot(), 0, offset);
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void TreeLayout::Firstwalk(TreeNode *node, int level)
{
	m_modifier[node] = 0.0f;
	m_prelim[node] = 0.0f;
	if(m_tree->IsLeafNode(node))
	{
		// determine preliminary coordinate
		if(m_tree->IsHasLeftSibling(node))
		{
			TreeNode *leftSibling = m_tree->GetLeftSibling(node);
			m_prelim[node] = m_prelim[leftSibling] +
							m_siblingSeparation +
							GetMeanNodeSize(leftSibling, node);
								
		}
		else
		{
			m_prelim[node] = 0.0f;
		}
	}
	// this node is not a leaf, call firstwalk method for each child.
	else
	{
		list<TreeNode *> children;
		m_tree->GetChildren(node, children);
		for(list<TreeNode *>::iterator it = children.begin(); it != children.end(); it++)
		{
			Firstwalk(*it, level + 1);
		}
		TreeNode *leftmost = children.front();
		TreeNode *rightmost = children.back();
		float midpoint = (m_prelim[leftmost] + m_prelim[rightmost]) / 2.0f;
		if(m_tree->IsHasLeftSibling(node))
		{
			TreeNode * leftSibling = m_tree->GetLeftSibling(node);
			m_prelim[node] = m_prelim[leftSibling] + m_siblingSeparation + 
				GetMeanNodeSize(leftSibling, node);
			m_modifier[node] = m_prelim[node] - midpoint;
			Apportion(node, level);
		}
		else 
		{
			m_prelim[node] = midpoint;
		}
	}
}

bool TreeLayout::Secondwalk(TreeNode *node, int level, int modsum)
{
	bool result = true;
	float xTemp = m_xTopAdjustment + level * m_levelSeparation;
	float yTemp = m_yTopAdjustment + m_prelim[node] + modsum;
	
	if(IsCheckExtentsRange(xTemp, yTemp))
	{
		m_tree->SetPosition(node, xTemp, yTemp);
		if(!m_tree->IsLeafNode(node))
		{
			list<TreeNode *> children;
			m_tree->GetChildren(node, children);
			for(list<TreeNode *>::iterator it = children.begin(); it != children.end(); it++)
			{
				result = Secondwalk(*it, level + 1, modsum + m_modifier[node]);
				if(!result)
				{
					break;
				}
			}
		}
	}
	else
	{
		result = false;
	}
	return result;
}


void TreeLayout::Apportion(TreeNode *node, int level)
{
	TreeNode *leftmost = m_tree->GetFirstChild(node);
	TreeNode *neighbor = m_tree->GetLeftNeighbor(leftmost);
	int depthToStop = m_tree->GetMaxDepth() - level;
	int compareDepth = 1;

	while(leftmost != NULL && neighbor != NULL && compareDepth <= depthToStop)
	{
		float leftModsum = 0;
		float rightModsum = 0;
		TreeNode *ancestorLeftmost = leftmost;
		TreeNode *ancestorNeighbor = neighbor;
		for(int i=0; i<compareDepth; i++)
		{
			ancestorLeftmost = m_tree->GetParent(ancestorLeftmost);
			ancestorNeighbor = m_tree->GetParent(ancestorNeighbor);
			rightModsum += m_modifier[ancestorLeftmost];
			leftModsum += m_modifier[ancestorNeighbor];
		}
		float moveDistance = (m_prelim[neighbor] + leftModsum + m_subtreeSeparation +
			GetMeanNodeSize(leftmost, neighbor)) - (m_prelim[leftmost] + rightModsum);
		
		if(moveDistance > 0)
		{
			// count interior sibling subtrees in leftSiblings
			TreeNode *tempNode = node;
			int leftSiblings = 0;
			while(tempNode != NULL && tempNode != ancestorNeighbor)
			{
				leftSiblings += 1;
				tempNode = m_tree->GetLeftSibling(tempNode);
			}
			if(tempNode != NULL)
			{
				float portion = moveDistance / leftSiblings;
				tempNode = node;
				while(tempNode != ancestorNeighbor)
				{
					m_prelim[tempNode] = m_prelim[tempNode] + moveDistance;
					m_modifier[tempNode] = m_modifier[tempNode] + moveDistance;
					moveDistance -= portion;
					tempNode = m_tree->GetLeftSibling(tempNode);
				}
			}
		}

		compareDepth += 1;
		if(m_tree->IsLeafNode(leftmost))
		{
			leftmost = m_tree->GetLeftmost(node, compareDepth);
			//neighbor = m_tree->GetLeftNeighbor(leftmost);
			//break;
		}
		else 
		{
			leftmost = m_tree->GetFirstChild(leftmost);
			//neighbor = m_tree->GetLeftNeighbor(leftmost);
		}
		neighbor = m_tree->GetLeftNeighbor(leftmost);
	}
}

float TreeLayout::GetMeanNodeSize(TreeNode *nodeOne, TreeNode *nodeTwo)
{
	return nodeOne->GetNodeRep().Height();
}

bool TreeLayout::IsCheckExtentsRange(float x, float y)
{
	return true;
}