/******************************************************************************\
*                                                                              *
* 1/4/2009                                                                     *
* NodeRep - Source File                                                        *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <NodeRep.h>
#include <QPainter>
#include <QRectF>
#include <QCursor>
#include <QPainterPath>
#include <TreeNodeRep.h>
#include <PreviewNodeRep.h>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER----------------------------------------------------------------------

bool NodeRep::s_nodeSizeChanged;
QPen NodeRep::s_pen;
QSize NodeRep::s_nodeSize;
QRectF NodeRep::s_nodeDrawRect;
QRectF NodeRep::s_titleRect;
QFont NodeRep::s_titleFont;
QPen NodeRep::s_titleFontPen;
QBrush NodeRep::s_nodeBrush;
QBrush NodeRep::s_nodeExploreBrush;
QBrush NodeRep::s_nodeInBrush;
QBrush NodeRep::s_nodeOutBrush;
QBrush NodeRep::s_nodeBiBrush;
QBrush NodeRep::s_nodeMovedToBrush;
QRectF NodeRep::s_nodeBoundRect;

//METHODS---------------------------------------------------------------------

//INIT
void NodeRep::Init()
{
	s_pen = QPen(QBrush(QColor(20,20,20)),1);
	s_nodeBrush = QBrush(QColor(245,250,245));
	s_nodeMovedToBrush = QBrush(QColor(130,130,130));

	//initialize nodeTitleFont
	SetNodeLabelFont(15,QColor(0,0,0));
	
	//initialize colors
	SetNodeColor(QColor(10,200,100),QColor(20,200,200),QColor(200,20,170),QColor(245,0,100));

	//initialize nodeSize
	SetNodeSize(300,40);
}

//SET NODE COLOR
void NodeRep::SetNodeColor( QColor &in, QColor &out, QColor bi, QColor sel )
{
	s_nodeInBrush = QBrush(in);
	s_nodeOutBrush = QBrush(out);
	s_nodeBiBrush = QBrush(bi);
	s_nodeExploreBrush = QBrush(sel);
}

//SET NODE LABEL COLOR
void NodeRep::SetNodeLabelFont(int fontSize, QColor &col)
{
	s_titleFont = QFont("Arial",fontSize);
	s_titleFontPen = QPen(col);
}

//SET NODE SIZE
void NodeRep::SetNodeSize( int width, int height )
{
	s_nodeSize.setWidth(width); s_nodeSize.setHeight(height);
	s_nodeDrawRect = QRectF(0,0,s_nodeSize.width(),s_nodeSize.height());
	s_titleRect = QRectF(20,0,s_nodeSize.width()-40,s_nodeSize.height());
	
	TreeNodeRep::OnNodeSizeSet();
	PreviewNodeRep::OnNodeSizeSet();
}

//GET NODE SIZE
QSize NodeRep::GetNodeSize()
{
	return QSize(s_nodeDrawRect.width(),s_nodeDrawRect.height());
}
/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

NodeRep::NodeRep( QGraphicsItem *parent /*= NULL*/ ) : QGraphicsItem(parent)
{
	m_eNodeType = eNORMAL;
	m_round = 10.0f;
	m_nodeTitle = "no title";
	OnCalcBoundRect();
}

NodeRep::~NodeRep()
{

}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//SET STATE
void NodeRep::SetState(ENodeState eNodeType)
{
	m_eNodeType = eNodeType;
}	

//GET STATE
NodeRep::ENodeState NodeRep::GetState()
{
	return m_eNodeType;
}

//SET TITLE
void NodeRep::SetTitle(QString &title)
{
	if(!title.isNull())
	{
		m_fullNodeTitle = title;
	}
	m_nodeTitle = QFontMetrics(s_titleFont).elidedText(m_fullNodeTitle,Qt::ElideLeft,OnGetTitleRect().width());
}

//GET TYPE COLOR
Vector3f NodeRep::GetTypeColor()
{
	return Vector3f(m_eNodeStateColor.red(),m_eNodeStateColor.green(),m_eNodeStateColor.blue());
}

//WIDTH
int NodeRep::Width()
{
	return s_nodeDrawRect.width();
}

//HEIGHT
int NodeRep::Height()
{
	return s_nodeDrawRect.height();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

//BOUNDING RECT
QRectF NodeRep::boundingRect() const
{
	return OnCalcBoundRect();
}

//PAINT
void NodeRep::paint( QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget /* = 0 */ )
{
	m_painter = painter;

	switch(m_eNodeType)
	{
		case eNORMAL:
		{
			m_drawBrush = s_nodeBrush;
			break;
		}
		case eEXPLORE:
		{
			m_drawBrush = s_nodeExploreBrush;
			break;
		}
		case eINGOING:
		{
			m_drawBrush = s_nodeInBrush;
			break;
		}
		case eOUTGOING:
		{
			m_drawBrush = s_nodeOutBrush;
			break;
		}
		case eBIDIRECTIONAL:
		{
			m_drawBrush = s_nodeBiBrush;
			break;
		}
		case eMOVEDTO:
		{
			m_drawBrush = s_nodeMovedToBrush;
			break;
		}
	}

	//store 
	m_eNodeStateColor = m_drawBrush.color();

	//set brush
	m_painter->setBrush(m_drawBrush);
}

//DRAW RECT
void NodeRep::DrawRect(QRectF &nodeDrawRect, float round)
{
	//draw node rect		
	m_painter->drawRoundedRect(nodeDrawRect,round,round);
}

//DRAW TEXT
void NodeRep::DrawText(QRectF &nodeTitleRect)
{
	//draw text
	m_painter->setPen(s_titleFontPen);
	m_painter->setFont(s_titleFont);
	m_painter->setRenderHint(QPainter::TextAntialiasing,true);
	m_painter->drawText(nodeTitleRect, Qt::AlignLeft | Qt::AlignVCenter, m_nodeTitle);
}

//ON CALC BOUNDING RECT
QRectF NodeRep::OnCalcBoundRect() const
{
	return s_nodeDrawRect;
}

//ON GET TITLE RECT
QRectF & NodeRep::OnGetTitleRect()
{
	return s_titleRect;
}



/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/