#ifndef NODEREP_H
#define NODEREP_H

/******************************************************************************\
*                                                                              *
* 1/4/2009                                                                     *
* NodeRep - Header File                                                        *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/
/******************************************************************************\
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX DESCRIPTION XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
\******************************************************************************/

//REQUIRED HEADERS 

#include <Definitions.h>
#include <Math/Vector2.h>
#include <Math/Vector3.h>
#include <QGraphicsItem>
#include <QColor>
#include <QBrush>
#include <QPen>

//DECLARATIONS

//PUBLIC DEFINITIONS TYPEDEFS

/******************************************************************************\
* CLASS                                                                        *
\******************************************************************************/
class NodeRep : public QGraphicsItem
{
	public:

		//CLASS STATIC DEFINITIONS

		enum ENodeState
		{
			eINGOING,
			eOUTGOING,
			eBIDIRECTIONAL,
			eNORMAL,
			eEXPLORE,
			eMOVEDTO
		};

		static void Init();
		static void SetNodeSize(int width, int height);
		static void SetNodeColor(QColor &in, QColor &out, QColor bi, QColor sel);
		static void SetNodeLabelFont(int fontSize, QColor &col);
		static QSize GetNodeSize();
		
		//METHODS

		NodeRep(QGraphicsItem *parent = NULL);
		virtual ~NodeRep();

		virtual int Width();
		virtual int Height();

		void SetState(ENodeState eNodeType);
		ENodeState GetState();
		void SetTitle(QString &title);
		QString & GetTitle() {return m_nodeTitle;}
		Vector3f GetTypeColor();
	
	protected:

		//CLASS STATIC DEFINITIONS

		static bool s_nodeSizeChanged;
		static QSize s_nodeSize;
		static QRectF s_nodeBoundRect;
		static QRectF s_nodeDrawRect;

		static QRectF s_titleRect;
		static QFont s_titleFont;
		static QPen s_titleFontPen;
		static QPen s_pen;

		static QBrush s_nodeBrush;
		static QBrush s_nodeExploreBrush;
		static QBrush s_nodeInBrush;
		static QBrush s_nodeOutBrush;
		static QBrush s_nodeBiBrush;
		static QBrush s_nodeMovedToBrush;

		//METHODS

		virtual QRectF & OnGetTitleRect();
		virtual QRectF OnCalcBoundRect() const;
		virtual QRectF boundingRect() const;
		virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget /* = 0 */);

		void DrawRect(QRectF &nodeDrawRect, float round);
		void DrawText(QRectF &nodeTitleRect);
			
		//MEMBER

		QPainter *m_painter;
		ENodeState m_eNodeType;
		QColor m_eNodeStateColor;
		QString m_nodeTitle;
		QString m_fullNodeTitle;
		QBrush m_drawBrush;
		float m_round;

	private:

}; 

#endif //NODEREP_H