/******************************************************************************\
*                                                                              *
* 19/4/2009                                                                    *
* PreviewNodeRep - Source File                                                 *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <PreviewNodeRep.h>
#include <QPainter>
#include <QRectF>
#include <QCursor>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

float PreviewNodeRep::m_sGlobalScale;
float PreviewNodeRep::m_sNodeHeightScale;
QRectF PreviewNodeRep::m_sScaledDrawRect;
QRectF PreviewNodeRep::m_sScaledTitleRect;
QRectF PreviewNodeRep::s_ownTitleRect;

//METHODS

void PreviewNodeRep::Init()
{
	m_sGlobalScale = 1.0f;
	m_sNodeHeightScale = 0.5f;

	OnNodeSizeSet();
}

void PreviewNodeRep::SetGlobalScale( float scale )
{
	m_sGlobalScale = 1.0f/scale;
}

float PreviewNodeRep::GetGlobalScale()
{
	return m_sGlobalScale;
}

float PreviewNodeRep::GetNodeHeightScale()
{
	return m_sNodeHeightScale;
}

void PreviewNodeRep::OnNodeSizeSet()
{
	int height = max<int>(20,s_nodeDrawRect.height()*m_sNodeHeightScale);
	m_sScaledDrawRect = QRectF(s_nodeDrawRect);
	m_sScaledDrawRect.setHeight(height);
	m_sScaledTitleRect = QRectF(s_titleRect);
	m_sScaledTitleRect.setHeight(height);
}

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

PreviewNodeRep::PreviewNodeRep()
{
	m_round = 5.0;
}

PreviewNodeRep::~PreviewNodeRep()
{

}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//WIDTH
int PreviewNodeRep::Width()
{
	return m_sScaledDrawRect.width();
}

//HEIGHT
int PreviewNodeRep::Height()
{
	return m_sScaledDrawRect.height();
}

//GET SCENE POS
QPointF PreviewNodeRep::GetScenePos()
{
	return this->scenePos();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

//PAINT
void PreviewNodeRep::paint( QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget /* = 0 */ )
{
	//paint default
	NodeRep::paint(painter,option,widget);
	
	//draw rect
	DrawRect(m_sScaledDrawRect,m_round);

	//draw text
	DrawText(m_sScaledTitleRect);
}

//ON CALC BOUNDING RECT
QRectF PreviewNodeRep::OnCalcBoundRect() const
{
	return s_nodeDrawRect;
}

//ON GET TITLE RECT
QRectF & PreviewNodeRep::OnGetTitleRect()
{
	return m_sScaledTitleRect;
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/