/******************************************************************************\
*                                                                              *
* 21/4/2009                                                                    *
* Preview - Source File                                                        *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <Preview.h>
#include <View.h>
#include <QGraphicsTextItem>
#include <QFont>
#include <algorithm>
#include <Memory/mmgr.h>
#include <QGraphicsSceneHoverEvent>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

#define PREVIEW_COLOR QColor(240,190,20)
#define MARGIN 4

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

Preview::Preview(View &view)
{
	//store
	m_pView = &view;

	//shadow
	m_pPeviewShadow = new QGraphicsRectItem(this);
	QLinearGradient linearGrad(QPointF(-10, 0), QPointF(0, 0));
	linearGrad.setColorAt(0, QColor(0,0,0,0));
	linearGrad.setColorAt(1, QColor(90,90,90,255));
	m_pPeviewShadow->setBrush(QBrush(linearGrad));
	m_pPeviewShadow->setPen(Qt::NoPen);
	m_pPeviewShadow->setParentItem(this);

	//init simple color
	setBrush(QBrush(PREVIEW_COLOR));

	//groups
	m_itemTextVisible = new QGraphicsTextItem("VISIBLE");
	m_itemTextVisible->setFont(QFont("Arial",15));
	m_itemTextInvisible = new QGraphicsTextItem("INVISILBLE");
	m_itemTextInvisible->setFont(QFont("Arial",15));
	m_visiblePreviewGroup = new QGraphicsItemGroup(this);
	m_nonVisiblePreviewGroup = new QGraphicsItemGroup(this);
	m_visiblePreviewGroup->addToGroup(m_itemTextVisible);
	m_nonVisiblePreviewGroup->addToGroup(m_itemTextInvisible);

	//clear
	ClearPreview();

	//adjust node size
	Adjust(NodeRep::GetNodeSize().width(),NodeRep::GetNodeSize().height());
	
	//init
	m_currentScroll = 0.0f;
}

Preview::~Preview()
{

}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//ADJUST
void Preview::Adjust(int newNodeWidth, int newNodeHeight)
{
	m_width = newNodeWidth + 2 * MARGIN;
	UpdateGeometry(*m_pView);
}

//UPDATE GEOMETRY
void Preview::UpdateGeometry(View &view)
{
	//set
	setRect(0,0,m_width,view.height());
	m_pPeviewShadow->setRect(-10,0,10,view.height());

	//update scroll data
	m_contentHeight = GetContentHeight();
	m_rangeHeight = view.height();

	float newMscroll = m_currentScroll-m_contentHeight+m_rangeHeight;
	Scroll(max<float>(newMscroll,m_currentScroll));

	//update cursor
	UpdateCursor();
}

//CREATE PREVIEW NODE
PreviewNodeRep & Preview::CreatePreviewNode( bool bGroupVisible )
{
	//new node
	PreviewNodeRep *newNode = new PreviewNodeRep();

	//node type
	if(!bGroupVisible)
	{
		int posY = m_nonVisiblePreviewGroup->children().size() * (newNode->Height() + 2) + 20;
		newNode->setPos(QPoint(MARGIN,posY));
		newNode->setParentItem(m_nonVisiblePreviewGroup);
		m_itemTextInvisible->setVisible(true);
	}
	else
	{
		int posY = m_visiblePreviewGroup->children().size() * (newNode->Height() + 2) + 20;
		newNode->setPos(QPoint(MARGIN,posY));
		newNode->setParentItem(m_visiblePreviewGroup);
		m_itemTextVisible->setVisible(true);
	}

	//set group positions
	m_nonVisiblePreviewGroup->setPos(0,m_currentScroll);
	if(m_itemTextInvisible->isVisible())
	{
		m_visiblePreviewGroup->setPos(0,m_nonVisiblePreviewGroup->childrenBoundingRect().height() + m_currentScroll);
	}
	else
	{
		m_visiblePreviewGroup->setPos(0,m_currentScroll);
	}

	//update contentheight
	m_contentHeight = GetContentHeight();

	//update cursor
	UpdateCursor();

	//return
	return *newNode;
}

//CLEAR PREVIEW
void Preview::ClearPreview()
{
	m_visiblePreviewGroup->removeFromGroup(m_itemTextVisible);
	m_nonVisiblePreviewGroup->removeFromGroup(m_itemTextInvisible);
		
	SAFE_DELETE(m_visiblePreviewGroup);
	SAFE_DELETE(m_nonVisiblePreviewGroup);
	
	m_visiblePreviewGroup = new QGraphicsItemGroup(this);
	m_nonVisiblePreviewGroup = new QGraphicsItemGroup(this);

	m_visiblePreviewGroup->addToGroup(m_itemTextVisible);
	m_itemTextVisible->setPos(20, 10);
	m_itemTextVisible->setVisible(false);

	m_nonVisiblePreviewGroup->addToGroup(m_itemTextInvisible);
	m_itemTextInvisible->setPos(20, 10);
	m_itemTextInvisible->setVisible(false);

	//updtae
	m_contentHeight = GetContentHeight();

	//reset
	Scroll(0);

	//update cursor
	UpdateCursor();
}

//GET PREVIEW NODE
PreviewNodeRep * Preview::GetPreviewNode( int id )
{
	for (int i = 0 ; i < m_visiblePreviewGroup->children().size() ; i++)
	{
		PreviewNodeRep *rep  = (PreviewNodeRep*) m_visiblePreviewGroup->children().at(i);
		if(rep->GetId() == id)
		{
			return rep;
		}
	}
	for (int i = 0 ; i < m_nonVisiblePreviewGroup->children().size() ; i++)
	{
		PreviewNodeRep *rep  = (PreviewNodeRep*) m_nonVisiblePreviewGroup->children().at(i);
		if(rep->GetId() == id)
		{
			return rep;
		}
	}
	return NULL;
}

void Preview::Scroll(float scroll)
{
	m_currentScroll = scroll;
	float minScroll = m_rangeHeight - m_contentHeight;

	//check
	if(m_currentScroll < minScroll)
	{
		m_currentScroll = minScroll;
	}
	if(m_currentScroll > 0)
	{
		m_currentScroll = 0;
	}
	
	//scroll nonVisibleGroup
	m_nonVisiblePreviewGroup->setPos(0,m_currentScroll);

	//scroll visibleGroup
	if(m_itemTextInvisible->isVisible())
	{
		m_visiblePreviewGroup->setPos(0,m_nonVisiblePreviewGroup->childrenBoundingRect().height() + m_currentScroll);
	}
	else
	{
		m_visiblePreviewGroup->setPos(0,m_currentScroll);
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

//MOUSE PRESS EVENT
static QPoint pressStart;
static float scrollStart;
void Preview::mousePressEvent( QGraphicsSceneMouseEvent *event )
{
	//remember
	pressStart = QPoint(event->scenePos().x(),event->scenePos().y());
	scrollStart = m_currentScroll;
}

//MOUSE MOVE EVENT
void Preview::mouseMoveEvent( QGraphicsSceneMouseEvent *event )
{
	//calculate
	QPoint pressEnd(event->scenePos().x(),event->scenePos().y());
	float dy = pressEnd.y() - pressStart.y();

	//scroll
	Scroll(scrollStart + dy);
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

//GET CONTENT HEIGHT
float Preview::GetContentHeight()
{
	return 
	(m_itemTextVisible->isVisible() ? m_visiblePreviewGroup->childrenBoundingRect().height() : 0) +
	(m_itemTextInvisible->isVisible() ? m_nonVisiblePreviewGroup->childrenBoundingRect().height() : 0) + 20;
}

//UPDATE CURSOR
void Preview::UpdateCursor()
{
	if(m_contentHeight > m_rangeHeight)
	{
		if(cursor().shape() != Qt::SizeVerCursor)
		{
			setCursor(Qt::SizeVerCursor);
		}
	}
	else
	{
		if(cursor().shape() != Qt::PointingHandCursor)
		{
			setCursor(Qt::PointingHandCursor);
		}
	}
}

 