/******************************************************************************\
*                                                                              *
* 1/4/2009                                                                     *
* TreeNodeRep - Source File                                                    *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <TreeNodeRep.h>
#include <Graph/TreeNode.h>
#include <Graph/Tree.h>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsScene>
#include <QPainter>
#include <QRectF>
#include <QCursor>
#include <QMenu>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

TreeNodeRep *TreeNodeRep::s_prevClick;
QPainterPath TreeNodeRep::s_infoPath;
QSize TreeNodeRep::s_infoSize;
QRectF TreeNodeRep::s_infoDrawRect;
QPoint TreeNodeRep::s_infoPathPos; 
QBrush TreeNodeRep::s_infoBrush;
QBrush TreeNodeRep::s_infoHoverBrush;
QPainterPath TreeNodeRep::s_arrowPath;
QPainterPath TreeNodeRep::s_doubleArrowPath;
QPen TreeNodeRep::s_arrowPen;
bool TreeNodeRep::s_bDrawArrows;
bool TreeNodeRep::s_bDrawInfoFields;
QRectF TreeNodeRep::s_ownTitleRect;

//METHODS

void TreeNodeRep::Init()
{
	s_prevClick = NULL;
	s_infoBrush = QBrush(QColor(190,194,190));
	s_infoHoverBrush = QBrush(QColor(210,215,210));

	s_infoPath.moveTo(0,0);
	s_infoPath.lineTo(5,-5);
	s_infoPath.lineTo(10,-5);
	s_infoPath.lineTo(0,5);
	s_infoPath.lineTo(-10,-5);
	s_infoPath.lineTo(-5,-5);
	s_infoPath.lineTo(0,0);

	s_arrowPen = QPen(QColor(0,0,0),2);
	s_arrowPath.moveTo(-7,0);
	s_arrowPath.lineTo(0,8);
	s_arrowPath.lineTo(0,2);
	s_arrowPath.lineTo(15,2);
	s_arrowPath.lineTo(15,-2);
	s_arrowPath.lineTo(0,-2);
	s_arrowPath.lineTo(0,-8);
	s_arrowPath.lineTo(-7,0);

	s_doubleArrowPath.moveTo(-8,0);
	s_doubleArrowPath.lineTo(-3,8);
	s_doubleArrowPath.lineTo(-3,2);
	s_doubleArrowPath.lineTo(3,2);
	s_doubleArrowPath.lineTo(3,8);
	s_doubleArrowPath.lineTo(8,0);
	s_doubleArrowPath.lineTo(3,-8);
	s_doubleArrowPath.lineTo(3,-2);
	s_doubleArrowPath.lineTo(-3,-2);
	s_doubleArrowPath.lineTo(-3,-8);
	s_doubleArrowPath.lineTo(-8,0);

	//set draw arrow and infofield components
	SetComponents(true,true);
}

void TreeNodeRep::OnNodeSizeSet()
{
	s_infoSize = QSize(60,s_nodeDrawRect.height());
	s_infoDrawRect = QRectF(s_nodeDrawRect.width() - s_infoSize.width(),0,s_infoSize.width(),s_infoSize.height());
	s_infoPathPos = QPoint(s_infoDrawRect.center().toPoint());

	if(s_bDrawInfoFields)
	{
		s_ownTitleRect = QRectF(20,0,s_nodeSize.width()- s_infoSize.width() - 20 ,s_nodeDrawRect.height());
	}
	else
	{
		s_ownTitleRect = QRectF(20,0,s_nodeSize.width() - 20,s_nodeDrawRect.height());
	}
}

void TreeNodeRep::SetComponents(bool bDrawArrows, bool bDrawInfoFields)
{
	s_bDrawArrows = bDrawArrows;
	s_bDrawInfoFields = bDrawInfoFields;
	OnNodeSizeSet();
}

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

TreeNodeRep::TreeNodeRep(QGraphicsItem *parent) : 
	m_bInfoVisible(false),
	m_hoverPenWidth(0),
	m_hoverInfoScale(1.0f),
	m_mutate(1.0f),
	m_heightScale(1.0f),
	m_mutateHeightScale(1.0f),
	NodeRep(parent)
{
	setAcceptHoverEvents(true);
	setCursor(QCursor(Qt::PointingHandCursor));
}

TreeNodeRep::~TreeNodeRep()
{
	scene()->removeItem(this);
}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//WIDTH
int TreeNodeRep::Width()
{
	return s_nodeDrawRect.width();
}

//HEIGHT
int TreeNodeRep::Height()
{
	return s_nodeDrawRect.height() * m_mutateHeightScale;
}

//SET TREE NODE
void TreeNodeRep::SetTreeNode(TreeNode *pTreeNode)
{
	m_pTreeNode = pTreeNode;
}

//GET TREE NODE
TreeNode * TreeNodeRep::GetTreeNode()
{
	return m_pTreeNode;
}

//SET HEIGHT SCALE
void TreeNodeRep::SetHeightScale( float heightScale )
{
	m_heightScale = heightScale;
}

//GET MUTATE HEIGHT SCALE
float TreeNodeRep::GetMutateHeightScale()
{
	return (m_mutate * 1.0) + ( (1.0f - m_mutate) * m_heightScale);
}

//MUTATE
void TreeNodeRep::Mutate(float dVal)
{
	m_mutate = dVal;
	m_mutateHeightScale = GetMutateHeightScale();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

//PAINT
void TreeNodeRep::paint( QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget /* = 0 */ )
{
	if(false)//m_bInfoVisible)
	{

	}
	
	//paint default
	NodeRep::paint(painter,option,widget);

	//check if mutating
	if(m_mutate < 1.0f)
	{
		//draw rect
		QRectF scaledDrawRect(s_nodeDrawRect);
		scaledDrawRect.setHeight(s_nodeDrawRect.height()*m_mutateHeightScale);
		DrawRect(scaledDrawRect, m_round);

		//draw title
		QRectF scaledTitleRect(s_ownTitleRect);
		scaledTitleRect.setHeight(s_ownTitleRect.height()*m_mutateHeightScale);
		DrawText(scaledTitleRect);

		//check hover
		if(m_hoverPenWidth > 1)
		{
			painter->setBrush(QColor(0,0,0,0));
			painter->setPen(QPen(QColor(30,30,30),m_hoverPenWidth));
			painter->drawRoundedRect(scaledDrawRect,10,10);
		}

		//mutate
		painter->setOpacity(m_mutate);

		if(s_bDrawArrows)
		{
			if(m_eNodeType < eNORMAL)
			{
				//draw arrow
				DrawArrow();
			}
		}

		if(s_bDrawInfoFields)
		{
			//info rect
			QRectF scaledInfoDrawRect(s_infoDrawRect);
			scaledInfoDrawRect.setHeight(s_infoDrawRect.height()*m_mutateHeightScale);
			painter->setBrush(m_hoverInfoScale < 1.2f ? s_infoBrush : s_infoHoverBrush);
			painter->drawRoundedRect(scaledInfoDrawRect, 10, 10);

			//info arrows
			painter->setBrush(QColor(122, 163, 39));
			painter->setPen(s_pen);
			painter->translate(scaledInfoDrawRect.center().toPoint());
			painter->scale(m_hoverInfoScale,m_hoverInfoScale);

			if(m_hoverInfoScale > 1.0f)
			{	
				painter->translate(0,-2);
			}

			if(m_bInfoVisible)
			{
				painter->translate(0,5);
				painter->scale(1,-1);
			}

			painter->drawPath(s_infoPath);
			painter->translate(QPoint(0,8));
			painter->drawPath(s_infoPath);
		}
	}
	else
	{
		if(s_bDrawArrows)
		{
			if(m_eNodeType < eNORMAL)
			{
				//draw arrow
				DrawArrow();
			}
		}
		
		//draw rect
		DrawRect(s_nodeDrawRect, m_round);

		//draw title
		DrawText(s_titleRect);

		//check hover
		if(m_hoverPenWidth > 1)
		{
			painter->setBrush(QColor(0,0,0,0));
			painter->setPen(QPen(QColor(30,30,30),m_hoverPenWidth));
			painter->drawRoundedRect(s_nodeDrawRect,10,10);
		}

		if(s_bDrawInfoFields)
		{
			//info rect
			painter->setBrush(m_hoverInfoScale < 1.2f ? s_infoBrush : s_infoHoverBrush);
			painter->drawRoundedRect(s_infoDrawRect,10,10);

			//info arrows
			painter->setPen(s_pen);
			painter->setBrush(QColor(122, 163, 39));
			
			painter->translate(s_infoPathPos);
			painter->scale(m_hoverInfoScale*(Height()/40.0f),m_hoverInfoScale*(Height()/40.0f));

			if(m_hoverInfoScale > 1.0f)
			{	
				painter->translate(0,-2);
			}

			if(m_bInfoVisible)
			{
				painter->translate(0,5);
				painter->scale(1,-1);
			}

			painter->drawPath(s_infoPath);
			painter->translate(QPoint(0,8));
			painter->drawPath(s_infoPath);
		}
	}
}

//DRAW ARROW
void TreeNodeRep::DrawArrow()
{
	bool arrowRight = false;

	if(s_prevClick)
	{
		int exploreLevel = m_pTreeNode->GetTree().GetLevel(s_prevClick->GetTreeNode());
		int thisLevel = m_pTreeNode->GetTree().GetLevel(m_pTreeNode);

		if(thisLevel == exploreLevel || thisLevel < exploreLevel)
		{
			arrowRight = true;
		}
	}
	
	m_painter->save();
	m_painter->setPen(s_arrowPen);

	if(arrowRight)
	{
		m_painter->translate(Width()+10,Height()/2.0f);
		if(m_eNodeType == eINGOING)
		{
			m_painter->scale(-1,1);
		}
	}
	else
	{
		m_painter->translate(-10,Height()/2.0f);
		if(m_eNodeType == eOUTGOING)
		{
			m_painter->scale(-1,1);
		}
	}

	if(m_eNodeType == eBIDIRECTIONAL)
	{
		m_painter->drawPath(s_doubleArrowPath);
	}
	else
	{
		m_painter->drawPath(s_arrowPath);
	}
	
	m_painter->restore();
}

//MOUSE PRESS EVENT
void TreeNodeRep::mousePressEvent( QGraphicsSceneMouseEvent *event )
{
	switch(event->button())
	{
		case Qt::LeftButton:
		{
			if(s_infoDrawRect.contains(event->pos()))
			{
				m_bInfoVisible = !m_bInfoVisible;
				m_pTreeNode->OnNodeShapeChange();
				//cout<<"info"<<endl;
			}
			else
			{
				m_pTreeNode->OnNodeClick(s_prevClick);
				s_prevClick = this;
				//cout<<"explore"<<endl;
			}
			event->accept();
			break;
		}
	}
}

//MOUSE RELEASE EVENT
void TreeNodeRep::mouseReleaseEvent( QGraphicsSceneMouseEvent *event )
{
	switch(event->button())
	{
		case Qt::RightButton:
		{
			if(isUnderMouse())
			{
				QMenu popMenu;
				QAction *actionSetRoot = popMenu.addAction("Set root");
				if(popMenu.exec(QCursor::pos()) == actionSetRoot)
				{
					m_pTreeNode->GetTree().SetRootNode(m_pTreeNode);
					//cout << "set new root" << endl;
				}
				event->accept();
				break;
			}
		}
	}
}

//MOUSE DOUBLE CLICK EVENT
void TreeNodeRep::mouseDoubleClickEvent( QGraphicsSceneMouseEvent *event )
{
	//cout<<"double"<<endl;
	m_pTreeNode->OnNodeDoubleClick();
	event->accept();
	update();
}

//HOVER ENTER EVENT
void TreeNodeRep::hoverEnterEvent( QGraphicsSceneHoverEvent *event )
{
	m_hoverPenWidth = 3;
	m_pTreeNode->OnNodeHoverEnter();
	event->accept();
	update();
}

//HOVER LEAVE EVENT
void TreeNodeRep::hoverLeaveEvent( QGraphicsSceneHoverEvent *event )
{
	m_hoverPenWidth = 1;
	m_hoverInfoScale = 1.0;
	m_pTreeNode->OnNodeHoverLeave();
	event->accept();
	update();
}

//HOVER MOVE EVENT
void TreeNodeRep::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
	if(s_infoDrawRect.contains(event->pos()))
	{
		m_hoverInfoScale = 1.3f;
	}
	else
	{
		m_hoverInfoScale = 1.0f;
	}

	event->accept();
	update();
}

//ON CALC BOUNDING RECT
QRectF TreeNodeRep::OnCalcBoundRect() const
{
	return s_nodeDrawRect.adjusted(m_hoverPenWidth, m_hoverPenWidth, m_hoverPenWidth, m_hoverPenWidth);
}

//ON GET TITLE RECT
QRectF & TreeNodeRep::OnGetTitleRect()
{
	return s_ownTitleRect;
}

void TreeNodeRep::SetToolTip(int inLink, int outLink, int biLink)
{
	QString toolTip;
	toolTip.sprintf("In: %d\nOut: %d\nBi: %d", inLink, outLink, biLink);
	setToolTip(toolTip);
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/