#ifndef TREENODEREP_H
#define TREENODEREP_H

/******************************************************************************\
*                                                                              *
* 1/4/2009                                                                     *
* TreeNodeRep - Header File                                                    *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/
/******************************************************************************\
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX DESCRIPTION XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
\******************************************************************************/

//REQUIRED HEADERS 

#include <Definitions.h>
#include <NodeRep.h>

//DECLARATIONS

class TreeNode;

//PUBLIC DEFINITIONS TYPEDEFS

/******************************************************************************\
* CLASS                                                                        *
\******************************************************************************/
class TreeNodeRep : public NodeRep
{
	public:

		//CLASS STATIC DEFINITIONS

		static bool s_bDrawArrows;
		static bool s_bDrawInfoFields;
		static QRectF s_ownTitleRect;

		static void Init();
		static void OnNodeSizeSet();
		static void SetComponents(bool bDrawArrows, bool bDrawInfoFields);

		//METHODS

		TreeNodeRep(QGraphicsItem *parent = NULL);
		~TreeNodeRep();

		virtual int Width();
		virtual int Height();

		void SetTreeNode(TreeNode *pTreeNode);
		TreeNode * GetTreeNode();
		void Mutate(float dVal);
		float GetMutateHeightScale();
		void SetHeightScale(float heightScale);
		void SetToolTip(int inLink, int outLink, int biLink);
	
	protected:

		//CLASS STATIC DEFINITIONS

		static QSize s_infoSize;
		static QRectF s_infoDrawRect;
		static QPainterPath s_infoPath;
		static QPoint s_infoPathPos;
		static QBrush s_infoBrush;
		static QBrush s_infoHoverBrush;

		static QPen s_arrowPen;
		static QPainterPath s_arrowPath;
		static QPainterPath s_doubleArrowPath;
		
		//METHODS

		virtual QRectF & OnGetTitleRect();
		virtual QRectF OnCalcBoundRect() const;
		virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget /* = 0 */);
		virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
		virtual void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
		virtual void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
		virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
		virtual void hoverMoveEvent(QGraphicsSceneHoverEvent *event);
		virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);

		void DrawArrow();

	private:

		//CLASS STATIC DEFINITIONS

		static TreeNodeRep *s_prevClick;

		//MEMBER

		TreeNode *m_pTreeNode;
		int m_hoverPenWidth;
		float m_hoverInfoScale;
		bool m_bInfoVisible;
		float m_mutate;
		float m_heightScale;
		float m_mutateHeightScale;
}; 

#endif //TREENODEREP_H