/******************************************************************************\
*                                                                              *
* 26/5/2009                                                                    *
* TreePlusSettings - Source File                                               *
*                                                                              *
* (c) Robert Hausmair, Christoph Winklhofer                                    *
*                                                                              *
\******************************************************************************/

//REQUIRED HEADERS 

#include <TreePlusSettings.h>
#include <QPalette>
#include <QColorDialog>
#include <Memory/mmgr.h>

//NAMESPACE

using namespace std;

//PRIVATE DEFINITIONS TYPEDEFS

//

/****************************************************************************\
* STATIC                                                                     *
\****************************************************************************/

//MEMBER

//METHODS

/****************************************************************************\
* CD                                                                         *
\****************************************************************************/

TreePlusSettings::TreePlusSettings()
{
	m_ui.setupUi(this);

	//Palette
	QPalette palette;

	//NODE COLORS

	//ColIn
	palette.setColor(QPalette::Window,QColor(132,200,129));
	m_ui.ColIn->setPalette(palette);
	
	//ColOut
	palette.setColor(QPalette::Window,QColor(103,142,109));
	m_ui.ColOut->setPalette(palette);

	//ColBi
	palette.setColor(QPalette::Window,QColor(162,144,188));
	m_ui.ColBi->setPalette(palette);

	//ColSel
	palette.setColor(QPalette::Window,QColor(245,160,22));
	m_ui.ColSel->setPalette(palette);

	//APPLICATION COLORS

	//ColScene
	palette.setColor(QPalette::Window,QColor(219,222,227));
	m_ui.ColScene->setPalette(palette);

	//ColPrev
	palette.setColor(QPalette::Window,QColor(179,188,189));
	m_ui.ColPrev->setPalette(palette);

	//ColNodeLabels
	palette.setColor(QPalette::Window,QColor(0,0,0));
	m_ui.ColNodeLabels->setPalette(palette);
}

TreePlusSettings::~TreePlusSettings()
{

}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//GET DATA
void TreePlusSettings::GetData( SettingsData &data )
{
	data.m_bLayoutChange = LayoutChanged(data);
	data.m_colNodeIngoing = m_ui.ColIn->palette().color(QPalette::Window);
	data.m_colNodeOutgoing = m_ui.ColOut->palette().color(QPalette::Window);
	data.m_colNodeBidirectional = m_ui.ColBi->palette().color(QPalette::Window);
	data.m_colNodeSelected = m_ui.ColSel->palette().color(QPalette::Window);
	data.m_colSceneBg = m_ui.ColScene->palette().color(QPalette::Window);
	data.m_colPreviewBg = m_ui.ColPrev->palette().color(QPalette::Window);
	data.m_colNodeLabels = m_ui.ColNodeLabels->palette().color(QPalette::Window);
	data.m_animationDuration = m_ui.AnimationDuration->value();
	data.m_animateCamera = m_ui.AnimateCamera->isChecked();
	data.m_drawArrows = m_ui.DrawArrows->isChecked();
	data.m_drawInfofields = m_ui.DrawInfoField->isChecked();
	data.m_levelSpacing = m_ui.LevelSep->text().toInt();
	data.m_subtreeSpacing = m_ui.SubtreeSep->text().toInt();
	data.m_siblingSpacing = m_ui.SiblingSep->text().toInt();
	data.m_orderBy = m_ui.OrderBy->currentIndex();
	data.m_sortBy = m_ui.SortBy->currentIndex();
	data.m_fontSize = m_ui.TextSizeNodeLabels->currentText().toInt();
	data.m_nodeWidth = m_ui.NodeWidth->currentText().toInt();
	data.m_nodeHeight = m_ui.NodeHeight->currentText().toInt();
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

//LAYOUT CHANGED
bool TreePlusSettings::LayoutChanged(SettingsData &data)
{
	int result = 0;
	result |= (data.m_levelSpacing != m_ui.LevelSep->text().toInt());
	result |= data.m_subtreeSpacing != m_ui.SubtreeSep->text().toInt();
	result |= data.m_siblingSpacing != m_ui.SiblingSep->text().toInt();
	result |= data.m_orderBy != m_ui.OrderBy->currentIndex();
	result |= data.m_sortBy != m_ui.SortBy->currentIndex();
	result |= data.m_nodeWidth != m_ui.NodeWidth->currentText().toInt();
	result |= data.m_nodeHeight != m_ui.NodeHeight->currentText().toInt();
	return result;
}

/****************************************************************************\
* PRIVATE SLOTS                                                              *
\****************************************************************************/

//APPLY
void TreePlusSettings::apply(QAbstractButton *btn)
{
	if(btn == (QAbstractButton*)(m_ui.buttonBox->button(QDialogButtonBox::Apply)))
	{
		emit Apply();
	}
}

//SELECT COLOR
void TreePlusSettings::SelectColor()
{
	QFrame *spot;

	hide();
	
	if(sender() == m_ui.ColInB)
	{
		spot = m_ui.ColIn;
	}
	if(sender() == m_ui.ColOutB)
	{
		spot = m_ui.ColOut;
	}
	if(sender() == m_ui.ColBiB)
	{
		spot = m_ui.ColBi;
	}
	if(sender() == m_ui.ColSelB)
	{
		spot = m_ui.ColSel;
	}
	if(sender() == m_ui.ColSceneB)
	{
		spot = m_ui.ColScene;
	}
	if(sender() == m_ui.ColPrevB)
	{
		spot = m_ui.ColPrev;
	}
	if(sender() == m_ui.ColNodeLabelsB)
	{
		spot = m_ui.ColNodeLabels;
	}
	
	QColor newColor = QColorDialog::getColor(spot->palette().color(QPalette::Window));

	if(newColor.isValid())
	{
		//Palette
		QPalette palette;
		palette.setColor(QPalette::Window,newColor);
		spot->setPalette(palette);
	}

	show();
}